# @allmaps/triangulate

This module triangulates a polygon: it returns a set of small triangule that partition the polygon.

It is used in [@allmaps/render](../../packages/render/) to triangulate the mask of a georeferenced map into a set of triangles that can be rendered with WebGL.

## How it works

It uses a simple **constrained Delaunay triangulation algorithm** for polygons, built using [poly2tri.js](https://github.com/r3mi/poly2tri.js).

To learn more on how it works, check out this [Observable notebook](https://observablehq.com/@allmaps/a-simple-polygon-triangulation-algorithm).

## Installation

This is an ESM-only module that works in browsers and Node.js.

Install using npm:

```sh
npm install @allmaps/triangulate
```

## Usage

```js
import { triangulate } from '@allmaps/triangulate'

// polygons are not round-trip
const polygon = [
  [0.592, 0.953],
  [0.304, 2.394],
  [2.904, 2.201],
  [2.394, 0.232]
]

const distance = 1

// compute constrained triangulation of `polygon` using a mesh of size `distance`
const triangles = triangulate(polygon, distance)

// triangles = [
//   [
//     [1.3012562303117026, 2.3199729029037854],
//     [0.304, 2.394],
//     [1.304, 2.232]
//   ],
//   ...
// ]
```

## API

<!-- Generated by documentation.js. Update this documentation by updating the source code. -->

#### Table of Contents

*   [Triangulation](#triangulation)
    *   [triangulate](#triangulate)
        *   [Parameters](#parameters)
    *   [triangulatePoly2tri](#triangulatepoly2tri)
        *   [Parameters](#parameters-1)
*   [Types](#types)
    *   [Triangle](#triangle)
    *   [poly2tri.Triangle](#poly2tritriangle)
    *   [Ring](#ring)

### Triangulation



#### triangulate

Triangulates a polygon

##### Parameters

*   `polygon` **[Ring](#ring)** Polygon
*   `distance` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** Distance between the Steiner points placed in a grid inside the polygon

Returns **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)<[Triangle](#triangle)>** Array of triangles partitioning the polygon

#### triangulatePoly2tri

Triangulates a polygon (and returns the full Poly2tri output)

##### Parameters

*   `polygon` **[Ring](#ring)** Polygon
*   `distance` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** Distance between the Steiner points placed in a grid inside the polygon

Returns **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)<[poly2tri.Triangle](#poly2tritriangle)>** Array of triangles partitioning the polygon

### Types

The types used in this module are described below.

#### Triangle

Triangle as `[[x0, y0], [x1, y1], [x2, y2]]`

Type: [Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)

#### poly2tri.Triangle

Triangle object from [poly2tri](https://github.com/r3mi/poly2tri.js/) package

Type: [Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)

#### Ring

Polygon object, as an outer Ring only `[[number, number], ...]`

Type: [Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)

## Notes

### Stability

*   `poly2tri` doesn't allow self-intersection polygons and will raise an error for such inputs.
*   For certain polygon vertex configurations an especially for round numbers or small distance sizes, poly2tri is known to throw errors such as 'point collinearity' or 'pointerror'.

### Benchmark

For a 10 point polygon, here are some benchmarks for computing the triangulation with the distance as a fraction of the polygon's bbox diameter:

*   `triangulate(polygon, 1)`: 66719 ops/s to compute 8 triangles
*   `triangulate(polygon, bboxDiameter / 10)`: 10924 ops/s to compute 86 triangles
*   `triangulate(polygon, bboxDiameter / 40)`: 1115 ops/s to compute 1048 triangles
*   `triangulate(polygon, bboxDiameter / 100)`: 167 ops/s to compute 6216 triangles

See [`./bench/index.js`](`./bench/index.js`).

To run the benchmark, run `npm run bench`.
